/*
   FILNAME:			Text.h

   PURPOSE:			A class that is very similar to CString 

   CREATED BY:		Per Ghosh	
   ENVIRONEMENT:	Visual C++


INFO:

*/


#include "stdafx.h"

#include "Text.h"

// Instead of allways comparing the textpointer if NULL we set to some static data.
// This relives us from allocate memory with empty text.
static int     module_pintDummyData[] = { 0, 0, 0 };
static LPCTSTR module_pszDummyText = (LPCTSTR)(((BYTE*)&module_pintDummyData)+ sizeof( TXT::SSData ));

/************************************************
   *   Constructor/Destructor 
   *
 * * *
  ***
   *
*/

//-------------------------------------------------------------------------------------------------
CCText::CCText()
{
   m_pszText = const_cast<LPTSTR>( module_pszDummyText );
}

//-------------------------------------------------------------------------------------------------
CCText::CCText( LPCTSTR pszText, UINT uintBufferLength )
{
   UINT uintTextLength;

   // if no text and no length
   if( (pszText == NULL) && (uintBufferLength == 0) )
   {
      // set to our dummy text ( don't need to compare for m_pszText == NULL because of that)
      m_pszText = const_cast<LPTSTR>( module_pszDummyText );
      return;
   }
    
   if( pszText == NULL )                                    // if null just make the buffer
   {
      AllocateBuffer( uintBufferLength );
      return;
   }

   uintTextLength = lstrlen( pszText );

   // if length i zero and text has been set the make a buffer to the lengt of text
   if( uintBufferLength == 0 )
   {
      uintBufferLength = uintTextLength;
   }

   AllocateBuffer( uintBufferLength );                      // allocate buffer for text

   uintTextLength = min( uintTextLength, uintBufferLength );// set to text length

   // copy text
   CopyText( pszText, uintTextLength );
}

//-------------------------------------------------------------------------------------------------
CCText::CCText( const CCText& other )
{
   CopyConstructor( other );
}

//-------------------------------------------------------------------------------------------------
CCText& CCText::operator=( const CCText& other )
{
   if( &other == this ) return *this;

   CopyConstructor( other );

   return *this;
}

//-------------------------------------------------------------------------------------------------
void CCText::CopyConstructor( const CCText& other )
{
   TXT::SSData* pData = ((TXT::SSData*)m_pszText) - 1;

   if( other.m_pszText == module_pszDummyText )
   {
      m_pszText = const_cast<LPTSTR>( module_pszDummyText );
      return;
   }

   AllocateBuffer( pData->m_uintBufferLength );
   CopyText( other.m_pszText, pData->m_uintTextLength );
}

//-------------------------------------------------------------------------------------------------
CCText::~CCText()
{
   Release();
}

/************************************************
   *   Operator
   *
 * * *
  ***
   *
*/

/* ------------------------------------------------------------------------------------------------
 * OPERATOR:=
 * PURPOSE: Set new text
 */
const CCText& CCText::operator=( LPCTSTR pszText )
{
   UINT uintLength;

   uintLength = lstrlen( pszText );

   if( uintLength >= GetData()->m_uintBufferLength )
   {
      CCText::SafeDelete( m_pszText );
      AllocateBuffer( uintLength );
   }

   CopyText( pszText, uintLength );

   return *this;
}

/* ------------------------------------------------------------------------------------------------
 * OPERATOR:+=
 * PURPOSE: Adds a text string to the current text
 */
const CCText& CCText::operator+=( LPCTSTR pszText )
{
   ASSERT(pszText == NULL || ::AfxIsValidString( pszText, FALSE ) );
   Add( pszText );
	return *this;
}

/* ------------------------------------------------------------------------------------------------
 * OPERATOR:+=
 * PURPOSE: Convert number and add it to text
 */
const CCText& CCText::operator+=( long longNumber )
{
   TCHAR pszNumber[15];

   _ltot( longNumber, pszNumber, 10 );
   Add( pszNumber );
	return *this;
}


/* ------------------------------------------------------------------------------------------------
 * OPERATOR:<<
 * PURPOSE: Shifts characters to left
 */
const CCText& CCText::operator<<( UINT uintShift )
{
   if( uintShift > GetData()->m_uintTextLength )
   {
      Release();
      m_pszText = const_cast<LPTSTR>( module_pszDummyText );
   }
   else
   {
      GetData()->m_uintTextLength -= uintShift;
      memmove( m_pszText, m_pszText + uintShift, GetData()->m_uintTextLength + 1 );
   }

   return *this;
}

/************************************************
   *   Get/Set
   *
 * * *
  ***
   *
*/

/* ------------------------------------------------------------------------------------------------
 * NAME:    GetBuffer()
 * PURPOSE: Get pointer to the text to work with it directly 
 *
 * PARAM:   uintLength = minimal length of the buffer
 */
LPTSTR CCText::GetBuffer( UINT uintLength )
{
   TXT::SSData* pdataOld;

   pdataOld = NULL;

   if( uintLength > GetData()->m_uintBufferLength )
   {
      pdataOld = GetData();
      AllocateBuffer( uintLength );
      CopyText( pdataOld->Text(), pdataOld->m_uintTextLength );
   }

   if( pdataOld != NULL ) CCText::SafeDelete( pdataOld->Text() );

   return m_pszText;
}

/************************************************
   *   Functions (public)
   *
 * * *
  ***
   *
*/
/* ------------------------------------------------------------------------------------------------
 * NAME:    Add()
 * PURPOSE: Add a string to the current string
 *
 * PARAM:   pszText = the text that will be added to string
 */
void CCText::Add( LPCTSTR pszText )
{
   UINT uintLength;     // Length of the string we are adding

   uintLength = lstrlen( pszText );                         // get length 

   // if the buffer has enough room then just append the string
   if( GetData()->m_uintBufferLength > (uintLength + GetData()->m_uintBufferLength) )
   {
      Append( pszText, uintLength );
   }
   else
   {
      // create a new buffer and insert the new text
      NewText( m_pszText, pszText, 0 );
   }
}

void CCText::Release()
{
   ASSERT( m_pszText );

   if( m_pszText != module_pszDummyText )
   {
      delete [] (BYTE*)GetData();
   }
}

/* ------------------------------------------------------------------------------------------------
 * NAME:    Find()
 * PURPOSE: Find a string in text
 *
 * PARAM:   pszString = the string we are looking for
 *          uintCount = if we don't want to search the whole string then this marks when to stop
 */
int CCText::Find( LPCTSTR pszString, UINT uintCount )
{
   TCHAR  tcharSave;   // Save character if not searching whole text
   LPTSTR pszPosition; // Marks where the string was found

   tcharSave = _T('\0');                                    // no character saved

   if( uintCount && (uintCount < GetData()->m_uintTextLength) )
   {
      tcharSave = m_pszText[uintCount];                     // save character
      m_pszText[uintCount] = _T('\0');                      // terminate string
   }
                    
   pszPosition = _tcsstr( m_pszText, pszString );           // find string

   if( tcharSave != _T('\0') )                              // if character saved
   {
      m_pszText[uintCount] = tcharSave;                     // restore character
   }

   // return the index to string if found otherwise -1
   return (pszPosition == NULL ) ? -1 : ( pszPosition - m_pszText );
}



/* ------------------------------------------------------------------------------------------------
 * NAME:    
 * PURPOSE: 
 *
 * PARAM:   
 */
void CCText::AllocateBuffer( const UINT uintLength )
{
   ASSERT(uintLength <= INT_MAX-1);

   UINT uintTotalLength;

   TXT::SSData* pData;    // Pointer to information for the text

   // calculate total length
   uintTotalLength = sizeof( TXT::SSData ) + ((uintLength + 1) * sizeof( TCHAR ));

   // Allocate memory for the data for text and the requested length
   pData = (TXT::SSData*)new BYTE[uintTotalLength];

   m_pszText = pData->Text();                               // set pointer to text

   pData->m_uintBufferLength = uintLength;                  // set max length
   pData->m_uintTextLength = 0;                             // set current length

   m_pszText[0] = _T('\0');                                 // no characters yet so set first to 0
   m_pszText[uintLength] = _T('\0');                        // set last character in buffer to 0
}

/* ------------------------------------------------------------------------------------------------
 * NAME:    CopyText()
 * PURPOSE: 
 *
 * PARAM:   
 */
void CCText::CopyText( LPCTSTR pszText, UINT uintLength )
{
   if( uintLength == 0 )
   {
      uintLength = lstrlen( pszText );    
   }

   ASSERT( GetData()->m_uintBufferLength>=uintLength);

   memcpy( m_pszText, pszText, uintLength * sizeof( TCHAR ) );
   m_pszText[uintLength] = _T('\0');
   GetData()->m_uintTextLength = uintLength;
}

/************************************************
   *   Functions (protected)
   *
 * * *
  ***
   *
*/
/* ------------------------------------------------------------------------------------------------
 * NAME:    Append()
 * PURPOSE: Add a text to the curent text
 *
 * PARAM:   pszText    = the text that we are adding
 *          uintLength = length of the text
 */
void CCText::Append( LPCTSTR pszText, UINT uintLength )
{
   if( uintLength == 0 )
   {
      uintLength = lstrlen( pszText );    
   }

   ASSERT( GetData()->m_uintBufferLength >= (uintLength + GetData()->m_uintTextLength) );

   TCHAR* pszPosition;

   // get position after last character in string
   pszPosition = m_pszText + GetData()->m_uintTextLength;

   memcpy( pszPosition, pszText, uintLength * sizeof( TCHAR ) );
   pszPosition[uintLength] = _T('\0');

   GetData()->m_uintTextLength += uintLength;
}

/* ------------------------------------------------------------------------------------------------
 * NAME:    NewText()
 * PURPOSE: Allocates a new buffer and copies text into it. Deletes the old data for the object
 *
 * PARAM:   pszText1      = ponter to the first text that are added to text
 *          pszText2      = points to next text that are added
 *          uintSetLength = the length of the buffer (if 0 the buffer is equal to text)
 */
void CCText::NewText( LPCTSTR pszText1, LPCTSTR pszText2, UINT uintSetLength )
{
   UINT uintTextLength1;  // length of text 1
   UINT uintTextLength2;  // length of text 2
   UINT uintTotalLength;  // the length of bothe text 1 and text 2 or the SetLength

   TXT::SSData* pdataOld; // pointer to the data object for the text

   pdataOld = GetData();                                    // Old data

   uintTextLength1 = 0;
   uintTextLength2 = 0;

   if( uintSetLength != 0 )
   {
      uintTotalLength = uintSetLength;
   }
   else
   {
      if( pszText1 != NULL ) uintTextLength1 = lstrlen( pszText1 );
      
      if( pszText2 != NULL ) uintTextLength2 = lstrlen( pszText2 );

      uintTotalLength = uintTextLength1 + uintTextLength2;
   }

   if( uintTotalLength )
   {
      AllocateBuffer( uintTotalLength );
      if( pszText1 ) CopyText( pszText1, uintTextLength1 );

      if( pszText2 ) Append( pszText2, uintTextLength2 );
   }

   CCText::SafeDelete( pdataOld->Text() );
}

/* ------------------------------------------------------------------------------------------------
 * NAME:    SafeDelete()
 * PURPOSE: Delete a text from memory (Do not use this function to delete ordinary text pointers !)
 *
 * PARAM:   pszText = points to the text that we are deleting
 */
void CCText::SafeDelete( TCHAR* pszText )
{
   if( pszText != module_pszDummyText )
   {
      delete [] (BYTE*)(((TXT::SSData*)pszText) - 1);
   }
}