/*
   FILNAME:			Text.h

   PURPOSE:			A class that is very similar to CString 

   CREATED BY:		Per Ghosh	
   ENVIRONEMENT:	Visual C++


INFO:

*/

#ifndef __TEXT_H_
#define __TEXT_H_

namespace TXT
{
/* ==============================================
 * SSData
 * This structure has information for a string.
 * The main purpose is to be used in CCText for
 * containing the text. 
 * The format of the text in CCText is
 * <UINT><UINT><LPTSTR> in one memory block.
 * This is done for saving memory and make the 
 * class run faster.
 * CString works like this too.
 */
struct SSData
{
   UINT m_uintBufferLength;
   UINT m_uintTextLength;

   TCHAR* Text() { return (TCHAR*)(this+1); }
};       

}

class CCText
{
// Constructor/Destructor
public:
   CCText();
   CCText( const CCText& other );
   CCText( LPCTSTR pszText, UINT uintBufferLength = 0 );
   CCText& operator=( const CCText& other );
   void CopyConstructor( const CCText& other );

   ~CCText();
                                  
// Attributes
private:
   LPTSTR m_pszText;

// Get/Set
public:
   // Get the length of current text
   UINT   GetLength() { return GetData()->m_uintTextLength; }
   // Get pointer to buffer for direct modification
   LPTSTR GetBuffer( UINT uintLength );

// Operator
public:
	operator LPCTSTR() const { return m_pszText; }  // pointer to string
   const CCText& operator= ( LPCTSTR pszText );    // Set a new text
	const CCText& operator+=( LPCTSTR pszText );    // Add a text
	const CCText& operator+=( long longNumber );    // Add a number (converted to text)
   const CCText& operator<<( UINT uintShift );     // shift characters to left

// Operations
public:
   // Add text to the current text ( += )
   void Add( LPCTSTR pszText );
   // Find a string in current and return the index where found
   int  Find( LPCTSTR pszString, UINT uintCount = 0 );
   // Compare if two texts is equal
   int  Compare( LPCTSTR pszText ) const { return (_tcscmp( m_pszText, pszText ) == 0 ); }

   void Release();

protected:
   // Get pointer to the data for the text
   TXT::SSData* GetData() { ASSERT(m_pszText!=NULL); return ((TXT::SSData*)m_pszText) - 1; }

   void AllocateBuffer( const UINT uintLength );
   void CopyText( LPCTSTR pszText, UINT uintLength );
   void Append( LPCTSTR pszText, UINT uintLength );
   void NewText( LPCTSTR pszText1, LPCTSTR pszText2, UINT uintLength );

   static void SafeDelete( TCHAR* pszText );
};


inline BOOL operator==( const CCText& Text1, const CCText& Text2 )   // CCText == CCText
   { return _tcscmp( Text1, Text2 ) == 0; }
inline BOOL operator==( const CCText& Text, LPCTSTR pszText )        // CCText == LPCTSTR
   { return _tcscmp( Text, pszText ) == 0; }
inline BOOL operator==( LPCTSTR pszText, const CCText& Text  )       // LPCTSTR == CCText
   { return _tcscmp( pszText, Text ) == 0; }

inline BOOL operator!=( const CCText& Text1, const CCText& Text2 )   // CCText != CCText
   { return _tcscmp( Text1, Text2 ) != 0; }
inline BOOL operator!=( const CCText& Text, LPCTSTR pszText )        // CCText != LPCTSTR
   { return _tcscmp( Text, pszText ) != 0; }
inline BOOL operator!=( LPCTSTR pszText, const CCText& Text  )       // LPCTSTR != CCText
   { return _tcscmp( pszText, Text ) != 0; }

inline BOOL operator>( const CCText& Text1, const CCText& Text2 )    // CCText > CCText
   { return _tcscmp( Text1, Text2 ) > 0; }
inline BOOL operator>( const CCText& Text, LPCTSTR pszText )         // CCText > LPCTSTR
   { return _tcscmp( Text, pszText ) > 0; }
inline BOOL operator>( LPCTSTR pszText, const CCText& Text  )        // LPCTSTR > CCText
   { return _tcscmp( pszText, Text ) > 0; }

inline BOOL operator<( const CCText& Text1, const CCText& Text2 )    // CCText < CCText
   { return _tcscmp( Text1, Text2 ) < 0; }
inline BOOL operator<( const CCText& Text, LPCTSTR pszText )         // CCText < LPCTSTR
   { return _tcscmp( Text, pszText ) < 0; }
inline BOOL operator<( LPCTSTR pszText, const CCText& Text  )        // LPCTSTR < CCText
   { return _tcscmp( pszText, Text ) < 0; }

inline BOOL operator>=( const CCText& Text1, const CCText& Text2 )   // CCText >= CCText
   { return _tcscmp( Text1, Text2 ) >= 0; }
inline BOOL operator>=( const CCText& Text, LPCTSTR pszText )        // CCText >= LPCTSTR
   { return _tcscmp( Text, pszText ) >= 0; }
inline BOOL operator>=( LPCTSTR pszText, const CCText& Text  )       // LPCTSTR >= CCText
   { return _tcscmp( pszText, Text ) >= 0; }

inline BOOL operator<=( const CCText& Text1, const CCText& Text2 )   // CCText <= CCText
   { return _tcscmp( Text1, Text2 ) <= 0; }
inline BOOL operator<=( const CCText& Text, LPCTSTR pszText )        // CCText <= LPCTSTR
   { return _tcscmp( Text, pszText ) <= 0; }
inline BOOL operator<=( LPCTSTR pszText, const CCText& Text  )       // LPCTSTR <= CCText
   { return _tcscmp( pszText, Text ) <= 0; }

#ifdef __AFX_H__
inline BOOL operator==( const CCText& Text, const CString& string )  // CCText == CString
   { return _tcscmp( Text, string ) == 0; }
inline BOOL operator==( const CString& string, const CCText& Text  ) // CString == CCText
   { return _tcscmp( string, Text ) == 0; }

inline BOOL operator!=( const CCText& Text, const CString& string )  // CCText != CString
   { return _tcscmp( Text, string ) != 0; }
inline BOOL operator!=( const CString& string, const CCText& Text  ) // CString != CCText
   { return _tcscmp( string, Text ) != 0; }

inline BOOL operator>( const CCText& Text, const CString& string )   // CCText > CString
   { return _tcscmp( Text, string ) > 0; }
inline BOOL operator>( const CString& string, const CCText& Text  )  // CString > CCText
   { return _tcscmp( string, Text ) > 0; }

inline BOOL operator<( const CCText& Text, const CString& string )   // CCText < CString
   { return _tcscmp( Text, string ) < 0; }
inline BOOL operator<( const CString& string, const CCText& Text  )  // CString < CCText
   { return _tcscmp( string, Text ) < 0; }

inline BOOL operator>=( const CCText& Text, const CString& string )  // CCText >= CString
   { return _tcscmp( Text, string ) >= 0; }
inline BOOL operator>=( const CString& string, const CCText& Text  ) // CString >= CCText
   { return _tcscmp( string, Text ) >= 0; }

inline BOOL operator<=( const CCText& Text, const CString& string )  // CCText <= CString
   { return _tcscmp( Text, string ) <= 0; }
inline BOOL operator<=( const CString& string, const CCText& Text  ) // CString <= CCText
   { return _tcscmp( string, Text ) <= 0; }
#endif //__AFX_H__

#endif // __TEXT_H_